from .debug import rec_log


def parse_lightlevel(data: bytearray):

    for i in range(0, len(data), 10):
        ll = data[i : i + 10]

        address = int(ll[0])
        state = bool(ll[1])
        dim = int(ll[6])  # int.from_bytes(ll[5:7], "little")
        cover_position = int.from_bytes(ll[5:7], "little")

        rec_log(f"LIGHTLEVEL {state=} {dim=}", address)

        yield {
            "address": address,
            "state": state,
            "dim": dim,
            "cover_position": cover_position,
        }



""" All device types handled by pyplejd """

from enum import StrEnum

from .plejd_device import PlejdDevice, PlejdInput, PlejdOutput
from .plejd_button import PlejdButton
from .plejd_cover import PlejdCover
from .plejd_fellowship_follower import PlejdFellowshipFollower
from .plejd_light import PlejdLight
from .plejd_motion_sensor import PlejdMotionSensor
from .plejd_relay import PlejdRelay
from .plejd_scene import PlejdScene

__all__ = [
    "PlejdDeviceType",
    "PlejdDevice",
    "PlejdInput",
    "PlejdOutput",
    "PlejdButton",
    "PlejdCover",
    "PlejdFellowshipFollower",
    "PlejdLight",
    "PlejdMotionSensor",
    "PlejdRelay",
    "PlejdScene",
]


class PlejdDeviceType(StrEnum):
    LIGHT = "LIGHT"
    SWITCH = "RELAY"
    BUTTON = "SENSOR"
    MOTION = "MOTION"
    COVER = "COVERABLE"
    SCENE = "SCENE"
    UNKNOWN = "UNKNOWN"



import asyncio
import logging
import os
from datetime import datetime, timedelta
from typing import Callable
import time

from bleak import BleakClient, BleakError
from bleak.backends.device import BLEDevice
from bleak_retry_connector import establish_connection

from .crypto import auth_response, encrypt_decrypt
from . import ble_characteristics as gatt
from . import payload_encode
from .parse_data import parse_data
from .parse_lightlevel import parse_lightlevel
from .ble_characteristics import PLEJD_SERVICE

_LOGGER = logging.getLogger(__name__)
_CONNECTION_LOG = logging.getLogger("pyplejd.ble.connection")


class PlejdMesh:
    def __init__(self, manager):
        self.manager = manager
        self._seen_nodes: dict[BLEDevice, int] = {}
        self._expected_nodes = set()
        self._connectable_nodes = set()
        self._gateway_node = None
        self._crypto_key: bytearray = None
        self._client: BleakClient = None

        self._connect_listeners = set()
        self._state_listeners = set()

        self._ble_lock = asyncio.Lock()

    @property
    def connected(self):
        return self._client is not None

    def expect_device(self, BLEaddress: str, connectable=True):
        self._expected_nodes.add(BLEaddress.replace(":", "").upper())
        if connectable:
            self._connectable_nodes.add(BLEaddress.replace(":", "").upper())

    def see_device(self, node: BLEDevice, rssi: int) -> bool:
        _CONNECTION_LOG.debug(
            f"Saw device {node} (rssi: {rssi}, prev: {self._seen_nodes.get(node, -1e6)})"
        )
        new_device = node not in self._seen_nodes
        self._seen_nodes[node] = max(rssi, self._seen_nodes.get(node, -1e6))
        return new_device

    def set_key(self, key: str):
        self._crypto_key = key

    def _subscribe(self, set_: set, listener: Callable):
        set_.add(listener)

        def remover():
            if listener in set_:
                set_.remove(listener)

        return remover

    def _publish(self, set_: set, *args, **kwargs):
        for listener in set_:
            listener(*args, **kwargs)

    def subscribe_connect(self, listener: Callable):
        return self._subscribe(self._connect_listeners, listener)

    def subscribe_state(self, listener: Callable):
        return self._subscribe(self._state_listeners, listener)

    async def disconnect(self):
        if not self._client:
            return False
        try:
            await self._client.stop_notify(gatt.PLEJD_LASTDATA)
            await self._client.stop_notify(gatt.PLEJD_LIGHTLEVEL)
            await self._client.disconnect()
        except BleakError:
            pass
        self._client = None
        self._publish(self._connect_listeners, {"connected": False})

    async def connect(self):
        if self.connected:
            return True
        _CONNECTION_LOG.debug("Trying to connect to BLE mesh")

        def _disconnect(reason):
            _CONNECTION_LOG.debug("Disconected from BLE mesh (%s)", reason)
            self._client = None
            self._gateway_node = None
            self._publish(self._connect_listeners, {"connected": False})

        # Try to connect to nodes in order of decreasing RSSI
        filtered_nodes = dict(
            filter(
                lambda n: n[0].address.replace(":", "").upper()
                in self._connectable_nodes,
                self._seen_nodes.items(),
            )
        )
        sorted_nodes = dict(
            sorted(filtered_nodes.items(), key=lambda n: n[1], reverse=True)
        )

        _CONNECTION_LOG.debug(f"Expected nodes: {self._expected_nodes}")
        _CONNECTION_LOG.debug(f"Connectable expected nodes: {self._connectable_nodes}")

        _CONNECTION_LOG.debug(f"Seen nodes: {self._seen_nodes}")
        _CONNECTION_LOG.debug(f"Connectable, seen nodes: {filtered_nodes}")
        _CONNECTION_LOG.debug(f"Sorted by signal strength: {sorted_nodes}")

        if not sorted_nodes:
            _CONNECTION_LOG.debug(
                "Failed to connect to plejd mesh - No valid devices: %s (%s)",
                self._seen_nodes,
                self._connectable_nodes,
            )
            return False
        client = None
        for node in sorted_nodes:
            try:
                _CONNECTION_LOG.debug("Attempting to connect to %s", node)
                client = await establish_connection(
                    BleakClient, node, "plejd", _disconnect
                )

                if not await self._authenticate(client):
                    await client.disconnect()
                    continue
                self._gateway_node = node.address
                break

            except (BleakError, asyncio.TimeoutError) as e:
                _CONNECTION_LOG.warning("Failed to connect to %s: %s", node, str(e))

        else:
            _CONNECTION_LOG.warning(
                "Failed to connect to plejd mesh - %s", sorted_nodes
            )
            return False

        async def _lastdata_listener(_, lastdata: bytearray):

            data = encrypt_decrypt(self._crypto_key, self._gateway_node, lastdata)
            retval = parse_data(data)

            self._publish(self._state_listeners, retval)

            if "button" in retval:
                await self.poll_buttons()

        async def _lightlevel_listener(_, lightlevel: bytearray):
            for state in parse_lightlevel(lightlevel):
                self._publish(self._state_listeners, state)

        await client.start_notify(gatt.PLEJD_LASTDATA, _lastdata_listener)
        await client.start_notify(gatt.PLEJD_LIGHTLEVEL, _lightlevel_listener)
        self._client = client

        self._publish(self._connect_listeners, {"connected": True})
        await self.poll()
        return True

    async def poll(self):
        client = self._client
        if client is None:
            return
        _LOGGER.debug("Polling mesh for current state")
        await client.write_gatt_char(gatt.PLEJD_LIGHTLEVEL, b"\x01", response=True)

    async def poll_buttons(self):
        await self._write(payload_encode.request_button(self))

    async def ping(self):
        async with self._ble_lock:
            if not await self.connect():
                retval = False
            if await self._ping(self._client):
                await self.poll()
                retval = True
        if retval:
            await self.poll_buttons()
        return retval

    async def set_state(self, address: int, **state):
        payloads = payload_encode.set_state(self, address, **state)
        await self._write(payloads)

    async def activate_scene(self, index: int):
        payloads = payload_encode.trigger_scene(self, index)
        return await self._write(payloads)

    async def poll_time(self, address: int):
        client = self._client
        if client is None:
            return False
        payloads = payload_encode.request_time(self, address)
        await self._write(payloads)

        retval = await client.read_gatt_char(gatt.PLEJD_LASTDATA)
        data = encrypt_decrypt(self._crypto_key, self._gateway_node, retval)
        ts = int.from_bytes(data[5:9], "little")
        dt = datetime.fromtimestamp(ts)

        now = datetime.now() + timedelta(seconds=3600 * time.daylight)
        if abs(dt - now) > timedelta(seconds=60):
            _LOGGER.debug(f"Device {address} repported the wrong time {dt} ({now=})")
            return True
        return False

    async def broadcast_time(self):
        payloads = payload_encode.set_time(self)
        await self._write(payloads)

    async def _write(self, payloads):
        client = self._client
        if client is None:
            return False
        try:
            async with self._ble_lock:
                for payload in payloads:
                    _LOGGER.debug("Writing to plejd mesh: %s", payload.hex())
                    await self._client.write_gatt_char(
                        gatt.PLEJD_DATA, payload, response=True
                    )
        except (BleakError, asyncio.TimeoutError) as e:
            _LOGGER.warning("Writing to plejd mesh failed: %s", str(e))
            return False
        return True

    async def _ping(self, client):
        if client is None:
            return False
        try:
            ping = bytearray(os.urandom(1))
            _LOGGER.debug("Ping(%s)", int.from_bytes(ping, "little"))
            await client.write_gatt_char(gatt.PLEJD_PING, ping, response=True)
            pong = await client.read_gatt_char(gatt.PLEJD_PING)
            _LOGGER.debug("Pong(%s)", int.from_bytes(pong, "little"))
            if (ping[0] + 1) & 0xFF == pong[0]:
                return True
        except (BleakError, asyncio.TimeoutError) as e:
            _LOGGER.warning("Plejd mesh keepalive signal failed: %s", str(e))
        return False

    async def _authenticate(self, client: BleakClient):
        if client is None:
            return False
        try:
            _CONNECTION_LOG.debug("Authenticating with plejd mesh")
            await client.write_gatt_char(gatt.PLEJD_AUTH, b"\0x00", response=True)
            challenge = await client.read_gatt_char(gatt.PLEJD_AUTH)
            response = auth_response(self._crypto_key, challenge)
            await client.write_gatt_char(gatt.PLEJD_AUTH, response, response=True)
            if not await self._ping(client):
                _CONNECTION_LOG.debug("Authentication failed!")
                return False
            _CONNECTION_LOG.debug("Authentication successful")
            return True
        except (BleakError, asyncio.TimeoutError) as e:
            _CONNECTION_LOG.warning("Plejd mesh authentication failed: %s", str(e))
        return False



try:
    from pydantic.v1 import BaseModel
except ImportError:
    from pydantic import BaseModel


class Site(BaseModel):
    siteId: str
    title: str


class SiteListItem(BaseModel):
    site: Site
    plejdDevice: list[str]
    gateway: list
    hasRemoteControlAccess: bool
    sitePermission: dict



from .plejd_device import PlejdDevice


class PlejdFellowshipFollower(PlejdDevice):
    # Fellowshipfollowers are lights that are part of a group
    # E.g. several DWN-01 can be grouped with one device being the leader and the rest followers
    # They all should be registered as devices, so that the BLE connection to the mesh can be made
    # through any one of them.

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.outputType = None



from __future__ import annotations
import logging
from datetime import timedelta
from typing import TypedDict

from bleak_retry_connector import close_stale_connections

from .ble import PlejdMesh, PLEJD_SERVICE
from .cloud import PlejdCloudSite

from .errors import AuthenticationError, ConnectionError
from .interface import (
    outputDeviceClass,
    inputDeviceClass,
    sceneDeviceClass,
    DeviceTypes,
)


__all__ = [
    "PlejdManager",
    "get_sites",
    "verify_credentials",
    "DeviceTypes",
    "AuthenticationError",
    "ConnectionError",
    "PLEJD_SERVICE",
]

dt = DeviceTypes


get_sites = PlejdCloudSite.get_sites
verify_credentials = PlejdCloudSite.verify_credentials


class PlejdManager:
    def __init__(self, username: str, password: str, siteId: str):
        self.credentials = {
            "username": username,
            "password": password,
            "siteId": siteId,
        }

        self.mesh = PlejdMesh(self)
        self.devices: list[dt.PlejdDevice | dt.PlejdScene] = []
        self.cloud = PlejdCloudSite(**self.credentials)
        self.options = {}

    async def init(self, sitedata=None):
        await self.cloud.load_site_details(sitedata)

        self.mesh.set_key(self.cloud.cryptokey)

        self.mesh.subscribe_connect(self._update_connected)
        self.mesh.subscribe_state(self._update_device)

        LOGGER = logging.getLogger("pyplejd.device_list")

        LOGGER.debug("Output Devices:")
        for device in self.cloud.outputs:
            cls = outputDeviceClass(device)
            dev = cls(**device, mesh=self.mesh)
            LOGGER.debug(dev)
            self.devices.append(dev)
            self.mesh.expect_device(dev.BLEaddress, dev.powered)

        LOGGER.debug("Input Devices:")
        for device in self.cloud.inputs:
            cls = inputDeviceClass(device)
            dev = cls(**device, mesh=self.mesh)
            LOGGER.debug(dev)
            self.devices.append(dev)
            self.mesh.expect_device(dev.BLEaddress, dev.powered)

        LOGGER.debug("Scenes:")
        for scene in self.cloud.scenes:
            cls = sceneDeviceClass(scene)
            scn = cls(**scene, mesh=self.mesh)
            LOGGER.debug(scn)
            self.devices.append(scn)

    def add_mesh_device(self, device, rssi) -> bool:
        return self.mesh.see_device(device, rssi)

    async def close_stale(self, device):
        await close_stale_connections(device)

    @property
    def connected(self):
        return self.mesh is not None and self.mesh.connected

    @property
    def site_data(self):
        return self.cloud.details

    async def get_raw_sitedata(self):
        return await self.cloud.get_raw_details()

    def _update_connected(self, state):
        for d in self.devices:
            d.update_state(available=state["connected"])

    def _update_device(self, state):
        for d in self.devices:
            if d.match_state(state):
                d.update_state(**state)

    @property
    def ping_interval(self):
        return timedelta(minutes=10)

    async def ping(self):
        retval = await self.mesh.ping()
        return retval

    async def broadcast_time(self):
        for d in self.devices:
            if d.powered:
                if await self.mesh.poll_time(d.address):
                    await self.mesh.broadcast_time()
                    return

    async def disconnect(self):
        await self.mesh.disconnect()



from .debug import rec_log


def parse_data(data: bytearray):
    data_bytes = [data[i] for i in range(0, len(data))]
    data_hex = "".join(f"{b:02x}" for b in data_bytes)

    match data_bytes:
        case [0x01, 0x01, 0x10, *extra]:
            # Time data
            rec_log(f"TIME DATA {extra}", "TME")
            rec_log(f"    {data_hex}", "TME")

        case [0x02, 0x01, 0x10, *extra]:
            # Scene update
            rec_log(f"SCENE UPDATGE {extra}", "SCN")
            rec_log(f"    {data_hex}", "SCN")

        case [0x00, 0x01, 0x10, 0x00, 0x21, scene, *extra]:
            # Scene triggered
            rec_log(f"SCENE TRIGGER {scene=} {extra=}", "SCN")
            rec_log(f"    {data_hex}", "SCN")
            return {
                "scene": scene,
                "triggered": True,
            }

        case [0x00, 0x01, 0x10, 0x00, 0x15, *extra]:
            # Identify buttons command
            rec_log(f"IDENTIFY BUTTON REQUEST {extra=}")
            rec_log(f"    {data_hex}")

        case [0x00, 0x01, 0x10, 0x00, 0x16, addr, button, *extra]:
            # Button pressed
            rec_log(f"BUTTON {button=} {extra=}", addr)
            rec_log(f"    {data_hex}", addr)
            return {
                "address": addr,
                "button": button,
                "action": "release" if len(extra) and not extra[0] else "press",
            }

        case [addr, 0x01, 0x10, 0x00, 0xC8, state, dim1, dim2, *extra] | [
            addr,
            0x01,
            0x10,
            0x00,
            0x98,
            state,
            dim1,
            dim2,
            *extra,
        ]:
            # State dim command
            extra_hex = "".join(f"{e:02x}" for e in extra)
            rec_log(f"DIM {state=} {dim1=} {dim2=} {extra=} {extra_hex}", addr)

            dim = dim2
            cover_position = int.from_bytes(
                [dim1, dim2], byteorder="little", signed=True
            )
            cover_angle = None
            if extra:
                # The cover angle is given as a six bit signed number?
                cover_angle = extra[0]
                cover_angle_sign = 1
                if cover_angle & 0x20:
                    cover_angle = ~cover_angle
                    cover_angle_sign = -1
                cover_angle = (cover_angle & 0x1F) * cover_angle_sign

            rec_log(f"    {cover_position=} {cover_angle=}", addr)
            rec_log(f"    {data_hex}", addr)
            return {
                "address": addr,
                "state": state,
                "dim": dim,
                "cover_position": cover_position,
                "cover_angle": cover_angle,
            }

        case [addr, 0x01, 0x10, 0x00, 0x97, state, *extra]:
            # state command
            rec_log(f"STATE {state=} {extra=}", addr)
            rec_log(f"    {data_hex}", addr)
            return {
                "address": addr,
                "state": state,
            }

        case [addr, 0x01, 0x10, 0x04, 0x20, a, 0x01, 0x11, *color_temp]:
            # Color temperature
            color_temp = int.from_bytes(color_temp, "big")
            rec_log(f"COLORTEMP {a}-1-11 {color_temp=}", addr)
            rec_log(f"    {data_hex}", addr)
            return {
                "address": addr,
                "temperature": color_temp,
            }

        case [addr, 0x01, 0x10, 0x04, 0x20, a, 0x03, b, *extra, ll1, ll2]:
            # Motion
            lightlevel = int.from_bytes([ll1, ll2], "big")
            rec_log(f"MOTION {a}-3-{b} {extra=} {lightlevel=}", addr)
            rec_log(f"    {data_hex}", addr)
            return {
                "address": addr,
                "motion": True,
                "luminance": lightlevel,
            }

        case [addr, 0x01, 0x10, 0x04, 0x20, a, 0x05, *extra]:
            # Off by timeout?
            rec_log(f"TIMEOUT {a=}-5 {extra=}", addr)
            rec_log(f"    {data_hex}", addr)

        case [addr, 0x01, 0x10, 0x04, 0x20, *extra]:
            # Unknown new style command
            extra = [f"{e:02x}" for e in extra]
            rec_log(f"UNKNOWN NEW STYLE {addr=} {extra=}", addr)
            rec_log(f"    {data_hex}", addr)

        case [addr, 0x01, 0x10, cmd1, cmd2, *extra]:
            # Unknown command
            cmd = (f"{cmd1:x}", f"{cmd2:x}")
            extra = [f"{e:02x}" for e in extra]
            rec_log(f"UNKNONW OLD COMMAND {addr=} {cmd=} {extra=}", addr)
            rec_log(f"    {data_hex}", addr)

        case _:
            # Unknown command
            rec_log(f"UNKNOWN {data=}")
            rec_log(f"    {data_hex}")

    return {}



from __future__ import annotations
from enum import IntFlag
from ..cloud import site_details as sd

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..ble import PlejdMesh


class PlejdTraits(IntFlag):
    POWER = 0x8
    TEMP = 0x4
    DIM = 0x2
    GROUP = 0x1

    COVER = 0x10
    TILT = 0x40


class PlejdDevice:
    def __init__(
        self,
        address: int,
        deviceAddress: int,
        device: sd.Device,
        plejdDevice: sd.PlejdDevice,
        settings: sd.PlejdDeviceOutputSetting | sd.PlejdDeviceInputSetting,
        room: sd.Room,
        mesh: PlejdMesh,
        *_,
        **__,
    ):
        self.address = address
        self.deviceAddress = deviceAddress
        self.plejdDevice = plejdDevice
        self.settings = settings
        self.deviceData = device
        self.roomData = room

        self._mesh = mesh
        self._state = {}

        self._listeners = set()

        self.outputType = "UNKNOWN"
        self.identifier = None
        self.device_identifier = (plejdDevice.deviceId, device.objectId)
        self.capabilities = PlejdTraits(self.deviceData.traits)

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.BLEaddress} ({self.address}) {self.name} {self.outputType}-{self.capabilities!r}>"

    def match_state(self, state):
        if state.get("address") == self.address:
            return True
        return False

    def subscribe(self, listener):
        self._listeners.add(listener)

        def remover():
            if listener in self._listeners:
                self._listeners.remove(listener)

        return remover

    def parse_state(self, update, state):
        return state

    def update_state(self, **state):
        self._state.update(state)
        state = self.parse_state(state, self._state)
        for listener in self._listeners:
            listener(state)

    @property
    def BLEaddress(self):
        return self.deviceData.deviceId

    @property
    def powered(self):
        return PlejdTraits.POWER in self.capabilities

    @property
    def name(self):
        return self.deviceData.title

    @property
    def room(self):
        return self.roomData.title

    @property
    def hidden(self):
        return self.deviceData.hiddenFromRoomList

    @property
    def hardware(self):
        if self.plejdDevice.firmware.notes:
            return self.plejdDevice.firmware.notes.split()[0]
        return f"-UNKNOWN- ({self.plejdDevice.hardwareId})"

    @property
    def firmware(self):
        return self.plejdDevice.firmware.version


class PlejdOutput(PlejdDevice):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.outputType = self.deviceData.outputType
        self.settings: sd.PlejdDeviceOutputSetting
        self.identifier = (self.plejdDevice.deviceId, "O", str(self.settings.output))


class PlejdInput(PlejdDevice):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.settings: sd.PlejdDeviceInputSetting
        self.identifier = (self.plejdDevice.deviceId, "I", str(self.settings.input))

    def match_state(self, state):
        if "button" in state:
            if (
                state.get("address") == self.deviceAddress
                and state.get("button") == self.settings.input
            ):
                return True
            return False
        return super().match_state(state)



from .plejd_device import PlejdInput


class PlejdButton(PlejdInput):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.outputType = "SENSOR"

    @property
    def button_id(self):
        return self.settings.input

    def parse_state(self, update, state):
        state = {**state}
        self._state["action"] = None  # Don't save the action
        return state



from .plejd_device import PlejdOutput, PlejdTraits


class PlejdLight(PlejdOutput):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.dimmable = PlejdTraits.DIM in self.capabilities

        self.colortemp = None
        if PlejdTraits.TEMP in self.capabilities and (
            ct := self.settings.colorTemperature
        ):
            self.colortemp = [ct.minTemperature, ct.maxTemperature]

    def parse_state(self, update, state):
        available = state.get("available", False)
        return {
            "available": available,
            "state": bool(state.get("state", False)) if available else False,
            "dim": state.get("dim", 0) / 0xFF * 255,
            "colortemp": state.get("colortemp", None),
        }

    async def turn_on(self, dim=None, colortemp=None):
        if not self._mesh:
            return
        if dim is not None:
            dim = int(dim)
        if colortemp is not None:
            colortemp = int(1e6 / colortemp)

        await self._mesh.set_state(
            self.address, state=True, dim=dim, colortemp=colortemp
        )

    async def turn_off(self):
        if not self._mesh:
            return
        await self._mesh.set_state(self.address, state=False)



import asyncio
from .plejd_device import PlejdInput


class PlejdMotionSensor(PlejdInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.outputType = "MOTION"

        self.cooldown = None
        # Motion sensors seem to timeout at 25-35 seconds
        # by the Nyquist criteria, we need our timeout to be at least
        # twice that time in order not to significantly miss any events.
        self.timeout = 75

    def parse_state(self, update, state):
        state = {**state}
        if state.get("motion", False):
            if self.cooldown:
                self.cooldown()
                self.cooldown = None

            def _callback():
                self.update_state(motion=False)

            loop = asyncio.get_running_loop()
            self.cooldown = loop.call_at(loop.time() + self.timeout, _callback).cancel

        self._state["motion"] = None
        return state



from __future__ import annotations

from typing import TypedDict, Type

from . import device_type as DeviceTypes
from ..cloud import PlejdEntityData, PlejdSceneData

dt = DeviceTypes


def outputDeviceClass(device: PlejdEntityData) -> Type[dt.PlejdDevice]:

    if device["plejdDevice"].isFellowshipFollower:
        return dt.PlejdFellowshipFollower

    tpe = device["device"].outputType
    if tpe == "LIGHT":
        return dt.PlejdLight
    if tpe == "RELAY":
        return dt.PlejdRelay
    if tpe == "COVERABLE":
        return dt.PlejdCover

    return dt.PlejdDevice


def inputDeviceClass(device: PlejdEntityData) -> Type[dt.PlejdDevice]:
    if device["motion"]:
        return dt.PlejdMotionSensor
    return dt.PlejdButton
    return PlejdDevice


def sceneDeviceClass(device: PlejdSceneData) -> Type[dt.PlejdDevice]:
    return dt.PlejdScene



from .plejd_device import PlejdOutput, PlejdTraits


class PlejdRelay(PlejdOutput):

    def parse_state(self, update, state):
        available = state.get("available", False)

        return {
            "available": available,
            "state": state.get("state", False) if available else False,
        }

    async def turn_on(self):
        if not self._mesh:
            return
        await self._mesh.set_state(self.address, state=True)

    async def turn_off(self):
        if not self._mesh:
            return
        await self._mesh.set_state(self.address, state=False)



from __future__ import annotations
import binascii
from datetime import datetime, timedelta
import time

import typing

if typing.TYPE_CHECKING:
    from . import PlejdMesh

from .crypto import encrypt_decrypt
from .debug import send_log


def encode(mesh: PlejdMesh, payloads: list[str]):
    return [
        encrypt_decrypt(
            mesh._crypto_key,
            mesh._gateway_node,
            binascii.a2b_hex(payload.replace(" ", "")),
        )
        for payload in payloads
    ]


def hex_payload(payload):
    return "".join([f"{b:02x}" for b in binascii.a2b_hex(payload.replace(" ", ""))])


def set_state(mesh: PlejdMesh, address, **state):
    payloads = []

    if (st := state.get("state", None)) is not None:
        if st:
            if (dim := state.get("dim", None)) is not None:
                # Dim command
                # AA 0110 0098 01 DDDD
                # Dim level is passed twice to transform it from 1 byte to 2
                payloads.append(f"{address:02x} 0110 0098 01 {dim:02x}{dim:02x}")
                send_log(f"DIM command {hex_payload(payloads[-1])}", address)
            else:
                # State command on
                # AA 0110 0097 01
                payloads.append(f"{address:02x} 0110 0097 01")
                send_log(f"ON command {hex_payload(payloads[-1])}", address)
        else:
            # State command off
            # AA 0110 0097 00
            payloads.append(f"{address:02x} 0110 0097 00")
            send_log(f"OFF command {hex_payload(payloads[-1])}", address)

    if (ct := state.get("colortemp", None)) is not None:
        # Color temperature command
        # AA 0110 0420 030111 TTTT
        payloads.append(f"{address:02x} 0110 0420 030111 {ct:04x}")
        send_log(f"COLORTEMP command {hex_payload(payloads[-1])}", address)

    if (cover := state.get("cover", None)) is not None:
        if cover < 0:
            # Cover command stop
            # AA 0110 0420 030807 00
            payloads.append(f"{address:02x} 0110 0420 030807 00")
            send_log(f"COVER STOP command {hex_payload(payloads[-1])}", address)
        else:
            # Cover command position
            # AA 0110 0420 030827 01 PPPP
            payloads.append(f"{address:02x} 0110 0420 030827 01 {cover:04x}")
            send_log(f"COVER POSITION command {hex_payload(payloads[-1])}", address)

    return encode(mesh, payloads)


def trigger_scene(mesh: PlejdMesh, index):
    # Scene trigger command
    # 02 0110 0021 II
    payload = f"02 0110 0021 {index:02x}"
    send_log(f"SCENE command {hex_payload(payload)}", "SCN")
    return encode(mesh, [payload])


def set_time(mesh: PlejdMesh):
    now = datetime.now() + timedelta(seconds=3600 * time.daylight)
    now_bytes = int(now.timestamp()).to_bytes(5, "little")

    # Time broadcast command
    # 00 0110 001B TTTTTTTT

    # TODO: or is it
    # 01 0110 001B TTTTTTTT 01
    # payload = f"01 0110 001B {now_bytes.hex()} 01"

    payload = f"00 0110 001B {now_bytes.hex()}"
    send_log(f"SET TIME command {hex_payload(payload)}", "TME")
    return encode(mesh, [payload])


def request_time(mesh: PlejdMesh, address):
    # Request time report command
    # AA 0102 001b

    payload: f"{address:02x} 0102 001b"
    send_log(f"TIME REQUEST {hex_payload(payload)}", "TME")
    return encode(mesh, [payload])


def request_button(mesh: PlejdMesh):
    # Request button identification
    # 00 0110 0015

    payload = f"00 0110 0015"
    send_log(f"IDENTIFY BUTTON REQUEST {hex_payload(payload)}")
    return encode(mesh, [payload])



import logging


def send_log(message, address=None):
    L_ALL = logging.getLogger("pyplejd.ble.device.all")
    if address is not None:
        L = logging.getLogger(f"pyplejd.ble.device.{address}")
        L.debug(f"SEND {address}: {message}")
        L_ALL.debug(f"SEND {address}: {message}")
    else:
        L_ALL.debug(f"SEND: {message}")


def rec_log(message, address=None):
    L_ALL = logging.getLogger("pyplejd.ble.device.all")
    if address is not None:
        L = logging.getLogger(f"pyplejd.ble.device.{address}")
        L.debug(f"RECEIVE {address}: {message}")
        L_ALL.debug(f"RECEIVE {address}: {message}")
    else:
        L_ALL.debug(f"RECEIVE: {message}")



# BLE_UUID_SUFFIX = "6085-4726-be45-040c957391b5"
# PLEJD_SERVICE = f"31ba0001-{BLE_UUID_SUFFIX}"
# PLEJD_LIGHTLEVEL = f"31ba0003-{BLE_UUID_SUFFIX}"
# PLEJD_DATA = f"31ba0004-{BLE_UUID_SUFFIX}"
# PLEJD_LASTDATA = f"31ba0005-{BLE_UUID_SUFFIX}"
# PLEJD_AUTH = f"31ba0009-{BLE_UUID_SUFFIX}"
# PLEJD_PING = f"31ba000a-{BLE_UUID_SUFFIX}"

# LIGHT = "LIGHT"
# SENSOR = "SENSOR"
# MOTION = "MOTION"
# SWITCH = "RELAY"
# COVERABLE = "COVERABLE"
# UNKNOWN = "UNKNOWN"

# OUTPUT_TYPES = [LIGHT, SENSOR, MOTION, SWITCH, COVERABLE, UNKNOWN]


# class Device:
#     def __init__(self, name, type, dimmable=False, colortemp=False):
#         self.name = name
#         self.type = type
#         self.dimmable = dimmable
#         self.colortemp = colortemp


# HARDWARE = {
#     "0": Device("-unknown-", UNKNOWN),
#     "1": Device("DIM-01", LIGHT, dimmable=True),
#     "2": Device("DIM-02", LIGHT, dimmable=True),
#     "3": Device("CTR-01", LIGHT),
#     "4": Device("GWY-01", UNKNOWN),
#     "5": Device("LED-10", LIGHT, dimmable=True),
#     "6": Device("WPH-01", SENSOR),
#     "7": Device("REL-01", SWITCH),
#     "8": Device("SPR-01", SWITCH),
#     "10": Device("WRT-01", SENSOR),
#     "11": Device("DIM-01-2P", LIGHT, dimmable=True),
#     "12": Device("DAL-01", UNKNOWN),
#     "13": Device("Generic", LIGHT),
#     "14": Device("DIM-01-LC", LIGHT, dimmable=True),
#     "15": Device("DIM-02-LC", LIGHT, dimmable=True),
#     "16": Device("JAL-01", LIGHT, dimmable=True),
#     "17": Device("REL-01-2P", SWITCH),
#     "18": Device("REL-02", SWITCH),
#     "19": Device("EXT-01", UNKNOWN),
#     "20": Device("SPR-01", SWITCH),
#     "36": Device("LED-75", LIGHT, dimmable=True),
#     "70": Device("WMS-01", MOTION),
#     "103": Device("OUT-01", LIGHT, dimmable=True, colortemp=True),
#     "167": Device("DWN-01", LIGHT, dimmable=True, colortemp=True),
#     "199": Device("DWN-02", LIGHT, dimmable=True, colortemp=True),
# }

# HARDWARE_UNKNOWN = HARDWARE["0"]



try:
    from pydantic.v1 import BaseModel
except ImportError:
    from pydantic import BaseModel
from .. import const


# Parameters are read from my own test site data. There may be more or fewer parameters in some objects
# Most things not currently used by pyplejd in any way is currently commented out.


class PlejdObject(BaseModel):  # TODO
    # createdAt: str
    # updatedAt: str
    # ACL: dict
    objectId: str


class Pointer(BaseModel):
    __type = "Pointer"
    className: str
    objectId: str


class User(PlejdObject):
    profileName: str
    # isInstaller: bool = False
    email: str
    locale: str = "en"
    username: str
    # hasIntegration: bool = False
    # emailVerified: bool = True
    # profile: dict = {}
    # _failed_login_count: int = 0
    sessionToken: str = None


class Site(PlejdObject):
    # installers: list[str] = []
    title: str
    siteId: str
    version: int
    # plejdMesh: Pointer # Pointer to SiteData.plejdMesh
    # astroTable: dict
    # city: str
    # coordinates: dict
    # country: str
    # deviceAstroTable: dict
    # zipCode: str
    # previousOwners: list[str] = []


class PlejdMesh(PlejdObject):
    siteId: str
    plejdMeshId: str
    meshKey: str
    cryptoKey: str
    # site: Pointer # Pointer to SiteData.site


class Room(PlejdObject):
    siteId: str
    roomId: str
    title: str
    category: str
    # imageHash: int


class Scene(PlejdObject):
    title: str
    sceneId: str
    siteId: str
    hiddenFromSceneList: bool = False
    # settings: str = ""


class Device(PlejdObject):
    deviceId: str
    siteId: str
    title: str
    traits: int
    hiddenFromRoomList: bool = False
    roomId: str | None = ""
    hiddenFromIntegrations: bool = False
    outputType: str | None = None


class Firmware(PlejdObject):  # TODO
    notes: str
    # data: dict
    # metaData: dict
    # meshCommands: dict
    version: str
    # buildTime: int
    # firmwareApi: str


class PlejdDevice(PlejdObject):
    deviceId: str
    siteId: str
    # installer: Pointer
    # dirtyInstall: bool
    # dirtyUpdate: bool
    # dirtyClock: bool
    # dirtySettings: bool
    hardwareId: str
    faceplateId: str | None = "0"
    firmware: Firmware
    # coordinates: dict = None
    # predefinedLoad: dict = None
    # diagnostics: str
    isFellowshipFollower: bool = False


class PlejdDeviceInputSetting(PlejdObject):
    deviceId: str
    siteId: str
    input: int
    motionSensorData: dict | None
    buttonType: str = ""
    # dimSpeed: int = 0
    # doubleSidedDirectionButton: bool = False
    # singleClick: str | None = None
    # doubleClick: str | None = None


class PredefinedLoad(PlejdObject):
    loadType: str
    # title_en: str
    # description_en: str
    # title_sv: str
    # description_sv: str
    # titleKey: str
    # descriptionKey: str
    # predefinedLoadData: str
    # defaultDimCurve: dict
    # allowedDimCurves: dict


class ColorTemperature(BaseModel):
    minTemperature: int
    maxTemperature: int
    # slewRate: int
    # minTemperatureLimit: int
    # maxTemperatureLimit: int
    behavior: str
    # startTemperature: int


class CoverableSettings(BaseModel):
    # coverableMovementDirection: str
    # coverableTiltTime: int
    coverableTiltStart: int | None = None
    coverableTiltEnd: int | None = None
    # coverablePostRunTime: int
    # coverableCalibration: dict


class PlejdDeviceOutputSetting(PlejdObject):
    deviceId: str
    siteId: str
    output: int | None = None
    deviceParseId: str
    # dimMin: int
    # dimMax: int
    # dimStart: int
    dimCurve: str | None = None
    # outputSpeed: float
    # outputStartTime: int
    # curveRectification: bool
    # curveLogarithm: int
    # curveSinusCompensation: int
    # bootState: str
    predefinedLoad: PredefinedLoad | None = None
    colorTemperature: ColorTemperature | None = None
    coverableSettings: CoverableSettings | None = None
    # minimumRelayOffTime: int = None


class MotionSensor(PlejdObject):
    deviceId: str
    siteId: str
    input: int | None = None
    deviceParseId: str
    # dirty: bool
    # dirtyRemove: bool
    # active: bool


class SceneStep(PlejdObject):
    sceneId: str
    siteId: str
    deviceId: str
    state: str
    value: int
    # dirty: bool
    # dirtyRemoved: bool
    # output: int


class SitePermission(PlejdObject):
    siteId: str
    userId: str
    user: User
    locked: bool
    isOwner: bool
    isInstaller: bool
    isUser: bool
    site: Site


class SiteDetails(BaseModel):
    site: Site
    plejdMesh: PlejdMesh
    rooms: list[Room]
    scenes: list[Scene]
    devices: list[Device]
    plejdDevices: list[PlejdDevice]
    # gateways: list
    # resourceSets: list
    # timeEvents: list
    # sceneSteps: list[SceneStep]
    # astroEvents: list
    inputSettings: list[PlejdDeviceInputSetting]
    outputSettings: list[PlejdDeviceOutputSetting]
    motionSensors: list[MotionSensor]
    rxAddress: dict[str, dict[str, int]] | None
    # stateTimers: dict
    # sitePermission: SitePermission
    inputAddress: dict[str, dict[str, int]]
    outputAddress: dict[str, dict[str, int]]
    deviceAddress: dict[str, int]
    outputGroups: dict[str, dict[str, list[int]]]
    roomAddress: dict[str, int]
    sceneIndex: dict[str, int]
    deviceLimit: int

    def find_plejdDevice(self, deviceId: str) -> PlejdDevice:
        for d in self.plejdDevices:
            if d.deviceId == deviceId:
                return d

    def find_outputSettings(
        self, deviceId: str, output: int
    ) -> PlejdDeviceOutputSetting:
        for d in self.outputSettings:
            if d.deviceId == deviceId and d.output == output:
                return d

    def find_inputSettings(self, deviceId, input: int) -> PlejdDeviceInputSetting:
        for d in self.inputSettings:
            if d.deviceId == deviceId and d.input == input:
                return d

    def find_motionSensorData(self, deviceId: str | None, input: int) -> MotionSensor:
        for d in self.motionSensors:
            if d.deviceId == deviceId and d.input == input:
                return d

    def find_device(
        self, deviceId: str | None = None, objectId: str | None = None
    ) -> Device:
        for d in self.devices:
            if objectId is not None and d.objectId == objectId:
                return d
            if deviceId is not None and d.deviceId == deviceId:
                return d

    def find_room(self, roomId: str) -> Room:
        for r in self.rooms:
            if r.roomId == roomId:
                return r



from .plejd_device import PlejdOutput


class PlejdCover(PlejdOutput):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # settings.coverableSettings.coverableTiltStart
        # settings.coverableSettings.coverableTiltEnd
        self.previous_position = None

    def parse_state(self, update, state):
        available = state.get("available", False)
        moving = bool(state.get("state", 0))
        position = state.get("cover_position", 0) / 0x7FFF * 100
        opening = None
        if moving:
            opening = bool(position > self.previous_position)
        self.previous_position = position
        return {
            "available": available,
            "moving": bool(state.get("state", 0)),
            "position": state.get("cover_position", 0) / 0x7FFF * 100,
            "angle": state.get("cover_angle", 0) * 5,
            "opening": opening,
        }

    async def open(self):
        await self._mesh.set_state(self.address, cover=1 * 0xFFFF)

    async def close(self):
        await self._mesh.set_state(self.address, cover=0)

    async def stop(self):
        await self._mesh.set_state(self.address, cover=-1)

    async def set_position(self, position):
        await self._mesh.set_state(self.address, cover=int(position / 100 * 0xFFFF))



import binascii
import hashlib
import struct

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend


def encrypt_decrypt(key: str, addr: str, data: bytearray) -> bytearray:
    key = binascii.a2b_hex(key.replace("-", ""))
    addr = binascii.a2b_hex(addr.replace("-", "").replace(":", ""))[::-1]

    buf = addr + addr + addr[:4]

    ct = Cipher(algorithms.AES(bytearray(key)), modes.ECB(), backend=default_backend())
    ct = ct.encryptor()
    ct = ct.update(buf)

    output = b""
    for i, d in enumerate(data):
        output += struct.pack("B", d ^ ct[i % 16])
    return output


def auth_response(key: str, challenge: bytearray) -> bytearray:
    key = binascii.a2b_hex(key.replace("-", ""))

    k = int.from_bytes(key, "big")
    c = int.from_bytes(challenge, "big")
    intermediate = hashlib.sha256((k ^ c).to_bytes(16, "big")).digest()
    part1 = intermediate[:16]
    part2 = intermediate[16:]
    return bytearray([(a ^ b) for (a, b) in zip(part1, part2)])



from aiohttp import ClientSession
import aiohttp
import logging
from typing import Generator, TypedDict

from . import site_details as sd
from .site_list import SiteListItem

from ..errors import AuthenticationError, ConnectionError

_LOGGER = logging.getLogger(__name__)

API_APP_ID = "zHtVqXt8k4yFyk2QGmgp48D9xZr2G94xWYnF4dak"
API_BASE_URL = "https://cloud.plejd.com"
API_LOGIN_URL = "/parse/login"
API_SITE_LIST_URL = "/parse/functions/getSiteList"
API_SITE_DETAILS_URL = "/parse/functions/getSiteById"


headers = {
    "X-Parse-Application-Id": API_APP_ID,
    "Content-Type": "application/json",
}


class PlejdSiteSummary(TypedDict):
    title: str
    deviceCount: int
    siteId: str


class PlejdEntityData(TypedDict):
    address: int
    deviceAddress: int
    device: sd.Device
    plejdDevice: sd.PlejdDevice
    settings: sd.PlejdDeviceOutputSetting | sd.PlejdDeviceInputSetting
    room: sd.Room
    motion: bool | None


class PlejdSceneData(TypedDict):
    scene: sd.Scene
    index: int


async def _set_session_token(session: ClientSession, username: str, password: str):
    resp = await session.post(
        API_LOGIN_URL,
        json={"username": username, "password": password},
    )
    if resp.status != 200:
        data = await resp.json()
        if data.get("code", 0) == 101:
            raise AuthenticationError("Invalid username/password")
        else:
            _LOGGER.debug("Authentication failed for unknown reason. No internet?")
            raise ConnectionError
    data = await resp.json()
    user = sd.User(**data)
    session.headers["X-Parse-Session-Token"] = user.sessionToken


class PlejdCloudSite:
    def __init__(self, username: str, password: str, siteId: str, **_):
        self.username = username
        self.password = password
        self.siteId = siteId
        self.details: sd.SiteDetails = None
        self._details_raw: str | None = None

    @staticmethod
    async def verify_credentials(username, password) -> bool:
        async with ClientSession(base_url=API_BASE_URL, headers=headers) as session:
            await _set_session_token(session, username, password)
            return True

    @staticmethod
    async def get_sites(username: str, password: str) -> list[PlejdSiteSummary]:
        try:
            async with ClientSession(base_url=API_BASE_URL, headers=headers) as session:
                await _set_session_token(session, username, password)
                resp = await session.post(API_SITE_LIST_URL, raise_for_status=True)
                data = await resp.json()
                sites = [SiteListItem(**s) for s in data["result"]]
                return [
                    {
                        "siteId": site.site.siteId,
                        "title": site.site.title,
                        "deviceCount": len(site.plejdDevice),
                    }
                    for site in sites
                ]
        except aiohttp.ClientError as err:
            raise ConnectionError from err

    async def get_details(self) -> None:
        try:
            async with ClientSession(base_url=API_BASE_URL, headers=headers) as session:
                await _set_session_token(session, self.username, self.password)
                resp = await session.post(
                    API_SITE_DETAILS_URL,
                    params={"siteId": self.siteId},
                    raise_for_status=True,
                )
                data = await resp.json()
                self._details_raw = data["result"][0]
                self.details = sd.SiteDetails(**data["result"][0])
        except aiohttp.ClientError as err:
            raise ConnectionError from err

    async def load_site_details(self, backup=None) -> None:
        try:
            await self.get_details()
        except (AuthenticationError, ConnectionError) as err:
            if backup:
                _LOGGER.debug("Loading site data failed. Reverting to back-up.")
                self._details_raw = backup
                self.details = sd.SiteDetails(**backup)
            else:
                raise err

        _LOGGER.debug("Site data loaded")
        _LOGGER.debug(("Mesh Devices:", self.mesh_devices))

    async def get_raw_details(self) -> str | None:
        try:
            await self.get_details()
        except (AuthenticationError, ConnectionError):
            pass
        return self._details_raw

    @classmethod
    async def create(
        cls, username: str, password: str, siteId: str
    ) -> "PlejdCloudSite":
        self = PlejdCloudSite(username, password, siteId)
        await self.get_details()
        return self

    @property
    def cryptokey(self) -> str:
        if not self.details:
            raise RuntimeError("No site details have been fetched")
        return self.details.plejdMesh.cryptoKey

    @property
    def mesh_devices(self) -> set[str]:
        if not self.details:
            raise RuntimeError("No site details have been fetched")
        retval = set()
        for device in self.details.devices:
            retval.add(device.deviceId)
        return retval

    @property
    def outputs(self) -> Generator[PlejdEntityData, None, None]:
        details = self.details
        if not details:
            raise RuntimeError("No site details have been fetched")

        for deviceId, outputs in details.outputAddress.items():
            plejdDevice = details.find_plejdDevice(deviceId)
            deviceAddress = details.deviceAddress.get(deviceId)

            # Ignore grouped devices
            if plejdDevice.isFellowshipFollower:
                continue

            for output, address in outputs.items():
                output = int(output)

                settings = details.find_outputSettings(deviceId, output)
                if not settings:
                    continue

                device = details.find_device(objectId=settings.deviceParseId)

                room = details.find_room(device.roomId)

                yield {
                    "address": address,
                    "deviceAddress": deviceAddress,
                    "device": device,
                    "plejdDevice": plejdDevice,
                    "settings": settings,
                    "room": room,
                }

    @property
    def inputs(self) -> Generator[PlejdSceneData, None, None]:
        details = self.details
        if not details:
            raise RuntimeError("No site details have been fetched")

        for deviceId, inputs in details.inputAddress.items():
            plejdDevice = details.find_plejdDevice(deviceId)
            deviceAddress = details.deviceAddress.get(deviceId)

            for input, address in inputs.items():
                input = int(input)

                settings = details.find_inputSettings(deviceId, input)
                if not settings:
                    continue

                if motionSensor := details.find_motionSensorData(deviceId, input):
                    device = details.find_device(objectId=motionSensor.deviceParseId)
                else:
                    device = details.find_device(deviceId=settings.deviceId)

                room = details.find_room(device.roomId)

                yield {
                    "address": address,
                    "deviceAddress": deviceAddress,
                    "device": device,
                    "plejdDevice": plejdDevice,
                    "settings": settings,
                    "room": room,
                    "motion": bool(motionSensor),
                }

    @property
    def scenes(self) -> Generator[dict, None, None]:
        if not self.details:
            raise RuntimeError("No site details have been fetched")

        details = self.details
        for scene in details.scenes:
            yield {"scene": scene, "index": details.sceneIndex.get(scene.sceneId, -1)}



from __future__ import annotations
from ..cloud import site_details as sd

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..ble import PlejdMesh

import logging

_LOGGER = logging.getLogger(__name__)


class PlejdScene:
    def __init__(
        self,
        scene: sd.Scene,
        index: int,
        mesh: PlejdMesh,
    ):
        self.scene = scene
        self.index = index

        self._mesh = mesh
        self._state = {}

        self._listeners = set()

        self.outputType = "SCENE"
        self.identifier = self.scene.sceneId

    def __repr__(self):
        return f"<{self.__class__.__name__} ({self.index}) {self.name}>"

    def match_state(self, state):
        if state.get("scene") == self.index:
            return True
        return False

    def subscribe(self, listener):
        self._listeners.add(listener)

        def remover():
            if listener in self._listeners:
                self._listeners.remove(listener)

        return remover

    def update_state(self, **state):
        self._state.update(state)
        state = self._state
        for listener in self._listeners:
            listener(state)
        self._state["triggered"] = False

    async def activate(self):
        await self._mesh.activate_scene(self.index)

    @property
    def BLEaddress(self):
        return None

    @property
    def name(self):
        return self.scene.title

    @property
    def hidden(self):
        return self.scene.hiddenFromSceneList

    @property
    def powered(self):
        return False



from setuptools import find_packages, setup

MIN_PY_VERSION = "3.10"
PACKAGES = find_packages()
VERSION = "0.14.3"

setup(
    name="pyplejd",
    packages=PACKAGES,
    version=VERSION,
    description="A python library for communicating with Plejd devices via bluetooth",
    author="Thomas Lovén",
    author_email="thomasloven@gmail.com",
    license="MIT",
    url="https://github.com/thomasloven/pyplejd",
    download_url=f"https://github.com/thomasloven/pyplejd/archive/v{VERSION}.tar.gz",
    install_requires=["aiohttp", "bleak", "bleak_retry_connector", "pydantic"],
    keywords=["plejd", "bluetooth", "homeassistant"],
    python_requires=f">={MIN_PY_VERSION}",
)



BLE_UUID_SUFFIX = "6085-4726-be45-040c957391b5"
PLEJD_SERVICE = f"31ba0001-{BLE_UUID_SUFFIX}"
PLEJD_LIGHTLEVEL = f"31ba0003-{BLE_UUID_SUFFIX}"
PLEJD_DATA = f"31ba0004-{BLE_UUID_SUFFIX}"
PLEJD_LASTDATA = f"31ba0005-{BLE_UUID_SUFFIX}"
PLEJD_AUTH = f"31ba0009-{BLE_UUID_SUFFIX}"
PLEJD_PING = f"31ba000a-{BLE_UUID_SUFFIX}"
